from django.db import models
from . import mixins, values


class FernetEncryptedTextField(mixins.FernetEncryptedMixin, models.TextField):
    pass


class FernetEncryptedBigIntegerField(
    mixins.FernetEncryptedIntegerMixin, models.BigIntegerField
):
    pass


class FernetEncryptedIntegerField(
    mixins.FernetEncryptedIntegerMixin, models.IntegerField
):
    pass


class FernetEncryptedSmallIntegerField(
    mixins.FernetEncryptedIntegerMixin, models.SmallIntegerField
):
    pass


class FernetEncryptedIPAddressField(mixins.FernetEncryptedMixin, models.IPAddressField):
    pass


class FernetEncryptedNullBooleanField(
    mixins.FernetEncryptedBoolMixin, models.NullBooleanField
):
    pass


class FernetEncryptedBinaryField(mixins.FernetEncryptedMixin, models.BinaryField):
    pass


class FernetEncryptedUUIDField(mixins.FernetEncryptedMixin, models.UUIDField):
    def to_python(self, value):
        if not value:
            return value
        return values.UUIDFieldValue(value)


class FernetEncryptedCharField(mixins.FernetEncryptedMixin, models.CharField):
    def to_python(self, value):
        if not value:
            return value
        return values.TimeFieldValue(value)


class FernetEncryptedTimeField(mixins.FernetEncryptedTimeMixin, models.TimeField):
    def to_python(self, value):
        if not value:
            return value
        return values.TimeFieldValue(value)


class FernetEncryptedDateField(mixins.FernetEncryptedMixin, models.DateField):
    def to_python(self, value):
        if not value:
            return value
        return values.DateFieldValue(value)


class FernetEncryptedDateTimeField(
    mixins.FernetEncryptedTimeMixin, models.DateTimeField
):
    def to_python(self, value):
        if not value:
            return value
        return values.DateTimeFieldValue(value)


class FernetEncryptedEmailField(mixins.FernetEncryptedMixin, models.EmailField):
    pass


class FernetEncryptedBooleanField(mixins.FernetEncryptedBoolMixin, models.BooleanField):
    pass



import binascii

from . import exceptions
from django.conf import settings
from cryptography.fernet import Fernet, MultiFernet


class FernetCrypter:
    def __init__(self):
        self.string_encoding = getattr(settings, "STRING ENCODING", "utf-8")
        self.keys = getattr(settings, "DSF_ENCRYPTION_KEYS", None)
        if not self.keys:
            raise exceptions.NoKeysDefinedException()
        if type(self.keys) == list:
            if not self.check_keys(self.keys):
                raise exceptions.InvalidKeyException()
            self.fernet = MultiFernet([Fernet(key) for key in self.keys])
        elif type(self.keys) == bytes:
            if not self.check_key(self.keys):
                raise exceptions.InvalidKeyException()
            self.fernet = Fernet(self.keys)

    def check_keys(self, keys: list) -> bool:
        return all(self.check_key(_key) for _key in keys)

    def check_key(self, key: bytes) -> bool:
        try:
            Fernet(key)
        except binascii.Error:
            return False
        return True

    def encrypt_string(self, _object: str) -> str:
        data = _object.encode(self.string_encoding)
        return self.fernet.encrypt(data).decode(self.string_encoding)

    def encrypt_bytes(self, _object: bytes) -> bytes:
        return self.fernet.encrypt(_object)

    def decrypt_string(self, _object: str) -> str:
        data = _object.encode(self.string_encoding)
        return self.fernet.decrypt(data).decode(self.string_encoding)

    def decrypt_bytes(self, _object: bytes) -> bytes:
        # with suppress(InvalidToken):
        return self.fernet.decrypt(_object)

    def encrypt(self, _object):
        if isinstance(_object, bytes):
            return self.encrypt(_object)
        else:
            return self.encrypt_string(str(_object))

    def decrypt(self, _object):
        if isinstance(_object, bytes):
            return self.decrypt(_object)
        else:
            return self.decrypt_string(str(_object))



from . import values
from .crypters import FernetCrypter
from cryptography.fernet import InvalidToken


class FernetEncryptedMixin:
    def to_python(self, value):
        if not value:
            return value
        super_value = super(FernetEncryptedMixin, self).to_python(value)
        return values.FieldValue(super_value)

    def get_db_prep_save(self, value, connection):
        value = super(FernetEncryptedMixin, self).get_db_prep_save(value, connection)
        if not value:
            return value
        fernet_crypter = FernetCrypter()
        try:
            fernet_crypter.decrypt(value)
        except InvalidToken:
            return fernet_crypter.encrypt(value)
        return value

    def from_db_value(self, value, *args, **kwargs):
        return self.to_python(value)

    def get_internal_type(self):
        return "TextField"


class FernetEncryptedBoolMixin(FernetEncryptedMixin):
    def get_db_prep_save(self, value, connection):
        fernet_crypter = FernetCrypter()
        try:
            fernet_crypter.decrypt(value)
        except InvalidToken:
            if value is True:
                value = "1"
            elif value is False:
                value = "0"
            return fernet_crypter.encrypt(value)
        return value

    def to_python(self, value):
        if not value:
            return value
        return values.BooleanFieldValue(value)


class FernetEncryptedIntegerMixin(FernetEncryptedMixin):
    def to_python(self, value):
        if not value:
            return value
        return values.IntegerFieldValue(value)


class FernetEncryptedTimeMixin(FernetEncryptedMixin):
    def get_db_prep_save(self, value, connection):
        if not value:
            return value
        fernet_crypter = FernetCrypter()
        try:
            fernet_crypter.decrypt(value)
        except InvalidToken:
            value = fernet_crypter.encrypt(value)
        return value



from contextlib import suppress
from uuid import UUID
from datetime import time, date, datetime
from .crypters import FernetCrypter
from cryptography.fernet import InvalidToken


class FieldValue:
    def __init__(self, value):
        self.value = value

    def __repr__(self):
        return str(self.value)

    def _decrypt_value(self):
        if not self.value:
            return self.value
        crypter = FernetCrypter()
        with suppress(InvalidToken):
            value = crypter.decrypt(self.value)
        return value

    @property
    def decrypted(self):
        return self._decrypt_value()


class BooleanFieldValue(FieldValue):
    @property
    def decrypted(self):
        value = self._decrypt_value()
        if value == "None":
            return None
        return value == "1"


class IntegerFieldValue(FieldValue):
    @property
    def decrypted(self):
        return int(self._decrypt_value())


class UUIDFieldValue(FieldValue):
    @property
    def decrypted(self):
        return UUID(self._decrypt_value())


class TimeFieldValue(FieldValue):
    @property
    def decrypted(self):
        return time.fromisoformat(self._decrypt_value())


class DateFieldValue(FieldValue):
    @property
    def decrypted(self):
        return date.fromisoformat(self._decrypt_value())


class DateTimeFieldValue(FieldValue):
    @property
    def decrypted(self):
        return datetime.fromisoformat(self._decrypt_value())



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="django-symmetric-fields",
    version="0.0.3",
    author="Szczaleg",
    description="Symmetric encryption for model fields in Django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Szczaleg/django-symmetric-fields",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=["symmetricfields"],
    python_requires=">=3.7",
    install_requires=["cryptography"]
)